import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import SecureBlackbox.Cloud.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;


public class DownloadPropsDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField tbDestination;
	private final ButtonGroup buttonGroup = new ButtonGroup();
	private JTextField tbStartOffset;
	private JTextField tbSize;
	private JRadioButton rbDownloadHoleObject;
	private JRadioButton rbPartialDownload;
	private boolean isOk;
	private String saveFileName;
	private JLabel lblSaveCaption;
	private JLabel label1;
	private JLabel label2;
	protected String m_DestPath;
	protected boolean m_PartialDownload;
	protected long m_StartOffset;
	protected long m_Size;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			DownloadPropsDialog dialog = new DownloadPropsDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public DownloadPropsDialog() {
		setTitle("Download properties");
		setBounds(100, 100, 489, 228);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		{
			lblSaveCaption = new JLabel("Save \"Object 1\" to:");
			lblSaveCaption.setBounds(10, 11, 431, 14);
			contentPanel.add(lblSaveCaption);
		}
		
		tbDestination = new JTextField();
		tbDestination.setBounds(10, 26, 360, 20);
		contentPanel.add(tbDestination);
		tbDestination.setColumns(10);
		
		JButton btnBrowse = new JButton("Browse");
		btnBrowse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbDestination.setText(getSaveFileName());
			}
		});
		btnBrowse.setBounds(380, 25, 91, 23);
		contentPanel.add(btnBrowse);
		
		rbPartialDownload = new JRadioButton("Partial download");
		rbPartialDownload.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				TogglePartialOptions(rbPartialDownload.isSelected());
			}
		});
		buttonGroup.add(rbPartialDownload);
		rbPartialDownload.setBounds(10, 80, 191, 23);
		contentPanel.add(rbPartialDownload);

		rbDownloadHoleObject = new JRadioButton("Download whole object");
		rbDownloadHoleObject.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				TogglePartialOptions(rbPartialDownload.isSelected());
			}
		});
		buttonGroup.add(rbDownloadHoleObject);
		rbDownloadHoleObject.setBounds(10, 54, 220, 23);
		contentPanel.add(rbDownloadHoleObject);
				
		label1 = new JLabel("Start offset:");
		label1.setBounds(53, 110, 91, 14);
		contentPanel.add(label1);
		
		label2 = new JLabel("Size:");
		label2.setBounds(165, 110, 46, 14);
		contentPanel.add(label2);
		
		tbStartOffset = new JTextField();
		tbStartOffset.setBounds(53, 126, 86, 20);
		contentPanel.add(tbStartOffset);
		tbStartOffset.setColumns(10);
		
		tbSize = new JTextField();
		tbSize.setBounds(160, 126, 86, 20);
		contentPanel.add(tbSize);
		tbSize.setColumns(10);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
			            m_DestPath = tbDestination.getText().trim();
			            if (m_DestPath == "")
			            {
			                showMessage("Please specify the destination path");
			            } 
			            else 
			            {
			                m_PartialDownload = rbPartialDownload.isSelected();
			                if (m_PartialDownload)
			                {
			                    m_StartOffset = Long.parseLong(tbStartOffset.getText());
			                    m_Size = Long.parseLong(tbSize.getText());
			                }
			            }
					
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		rbDownloadHoleObject.setSelected(true);
	}

	public void showMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	private String getSaveFileName() {
		JFileChooser fc = new JFileChooser(saveFileName);
			    
	    int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	public void Prepare(TElFileDataStorageObject o) {
        saveFileName = o.getName();
        lblSaveCaption.setText("Save object '" + o.getName() + "' to: ");
        tbStartOffset.setText("0");
        tbSize.setText(o.getSize() + "");
        rbDownloadHoleObject.setSelected(true);
        TogglePartialOptions(false);	
	}

    private void TogglePartialOptions(boolean value)
    {
        label1.setEnabled(value);
        label2.setEnabled(value);
        tbStartOffset.setEnabled(value);
        tbSize.setEnabled(value);
    }

	public boolean isOK() {
		return isOk;
	}

	public long getStartOffset() {
		return m_StartOffset;
	}

	public long getSizeEx() {
		return m_Size;
	}

	public String getDestPath() {
		return tbDestination.getText();
	}

	public boolean getPartialDownload() {
		return rbPartialDownload.isSelected();
	}
}
