package com.eldos.ws;

import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.HTTPCommon.*;
import SecureBlackbox.HTTPClient.*;
import SecureBlackbox.SSLCommon.TSBCertificateValidateEvent;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class Main {

	private JFrame frmSe;
	
	private TElHTTPSClient httpClient = new TElHTTPSClient();
	private TElWebSocketClient wsClient;
	private WSThread thread;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main window = new Main();
					window.frmSe.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public Main() {
		initialize();
		
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frmSe = new JFrame();
		frmSe.setTitle("SecureBlackbox WebSocket Client Sample");
		frmSe.setBounds(100, 100, 516, 297);
		frmSe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmSe.getContentPane().setLayout(null);
		
		final JButton btnConnect = new JButton("Connect");
		btnConnect.setBounds(308, 11, 89, 23);
		frmSe.getContentPane().add(btnConnect);
		
		final JButton btnSend = new JButton("Send");
		btnSend.setEnabled(false);
		btnSend.setBounds(401, 223, 89, 23);
		frmSe.getContentPane().add(btnSend);
		
		final JButton btnDisconnect = new JButton("Disconnect");
		btnDisconnect.setEnabled(false);
		btnDisconnect.setBounds(401, 11, 89, 23);
		frmSe.getContentPane().add(btnDisconnect);
		
		final JTextArea textURL = new JTextArea();
		textURL.setText("ws://echo.websocket.org");
		textURL.setBounds(10, 15, 288, 16);
		frmSe.getContentPane().add(textURL);
		
		final JTextPane textLog = new JTextPane();
		textLog.setBounds(10, 42, 480, 174);
		frmSe.getContentPane().add(textLog);
		
		final JTextArea textMsg = new JTextArea();
		textMsg.setText("Test message");
		textMsg.setBounds(10, 227, 379, 16);
		frmSe.getContentPane().add(textMsg);
		
		btnDisconnect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				thread.interrupt();
				
				btnDisconnect.setEnabled(false);
				btnConnect.setEnabled(true);
				btnSend.setEnabled(false);
			}
		});
		
		btnSend.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				wsClient.sendText(textMsg.getText());
				textLog.setText(textLog.getText() + "C->S: " + textMsg.getText() + "\n");
			}
		});
		
		btnConnect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				TSBWebSocketClientSession session = new TSBWebSocketClientSession();
				
				int res = httpClient.startWS(textURL.getText(), session);
				if (res != SBHTTPSClient.SB_WEBSOCKET_SUCCESS) {
					showMessage("Connection error #" + res);
					return;
				}
				
				wsClient = session.WSClient;
				
				thread = new WSThread(session.WSClient, session.Socket);
				thread.start();
				
				btnDisconnect.setEnabled(true);
				btnConnect.setEnabled(false);
				btnSend.setEnabled(true);
			}
		});
		
		final TSBWSTextDataEvent.Callback onTextData = new TSBWSTextDataEvent.Callback() {
			@Override
			public void tsbwsTextDataEventCallback(TObject sender, String text, boolean last) {
				textLog.setText(textLog.getText() + "S->C: " + text + "\n");
			}
		};
		
		final TSBHTTPBeforeWebSocketClientUsedEvent.Callback onBeforeWSClientUsed = new TSBHTTPBeforeWebSocketClientUsedEvent.Callback() {
			@Override
			public void tsbhttpBeforeWebSocketClientUsedEventCallback(TObject sender, TElWebSocketClient client, TElHTTPSClient httpClient) {
				// TODO Auto-generated method stub
				client.setOnTextData(new TSBWSTextDataEvent(onTextData));
			}
		};
		
		final TSBCertificateValidateEvent.Callback onCertValidate = new TSBCertificateValidateEvent.Callback() {
			@Override
			public void tsbCertificateValidateEventCallback(TObject sender, TElX509Certificate cert, TSBBoolean accept) {
				// For testing purposes only. Don't do this in production code!
				accept.value = true;
			}
		};
		
		httpClient.setOnBeforeWebSocketClientUsed(new TSBHTTPBeforeWebSocketClientUsedEvent(onBeforeWSClientUsed));
		httpClient.setOnCertificateValidate(new TSBCertificateValidateEvent(onCertValidate));
	}
	
	private void showMessage(String msg) {
    	JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.INFORMATION_MESSAGE);
	}
	
	private class WSThread extends java.lang.Thread {
		
		TElWebSocketClient wsClient;
		TElSocket socket;
		
		public WSThread(TElWebSocketClient wsClient, TElSocket socket) {
			this.wsClient = wsClient;
			this.socket = socket;
		}
		
		@Override
		public void run() {
			while (( ! this.isInterrupted()) && (socket.getState() == SBSocket.issConnected)) {
				try {
					if (socket.canReceive(500))
						wsClient.dataAvailable();
					else
						sleep(50);
				} catch (Exception ex) {
					break;
				}
			}
				
			try {
				wsClient.close(false);
					
				while (wsClient.getActive() && (socket.getState() == SBSocket.issConnected))
					wsClient.dataAvailable();
					
				socket.close(false);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}
}
