import java.awt.BorderLayout;

import javax.swing.DefaultListModel;
import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import java.awt.Dimension;
import javax.swing.JList;
import javax.swing.JButton;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import SecureBlackbox.Base.SBConstants;
import SecureBlackbox.Base.SBStrUtils;
import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.TElCustomCertStorage;
import SecureBlackbox.Base.TElRelativeDistinguishedName;
import SecureBlackbox.Base.TElX509Certificate;
import SecureBlackbox.MIME.TElMessagePart;
import SecureBlackbox.SMIME.TElMessagePartHandlerSMime;

public class MimeViewer_SMime extends MimeViewer_PlugControl {
	
	private static final long serialVersionUID = 1L;

	private JPanel tsSignInfo;
	private JPanel tsErrorInfo;
	private JPanel tsCryptInfo;
	private JPanel pBtns;
	private JList<String> lbxCertificates;
	private JScrollPane scrollPane;
	private JTextArea mErr;

	public MimeViewer_SMime() {
		setLayout(new BorderLayout(0, 0));
		
		PageControl = new JTabbedPane(JTabbedPane.TOP);
		add(PageControl);
		
		tsSignInfo = new JPanel();
		PageControl.addTab("Sign details", null, tsSignInfo, null);
		
		tsCryptInfo = new JPanel();
		PageControl.addTab("Encrypted Details", null, tsCryptInfo, null);
		tsCryptInfo.setLayout(new BorderLayout(0, 0));
		
		pBtns = new JPanel();
		pBtns.setPreferredSize(new Dimension(170, 10));
		tsCryptInfo.add(pBtns, BorderLayout.EAST);
		pBtns.setLayout(null);
		
		JButton btnViewDetails = new JButton("View Details");
		btnViewDetails.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				viewClick();
			}
		});
		btnViewDetails.setBounds(10, 11, 138, 23);
		pBtns.add(btnViewDetails);
		
		lbxCertificates = new JList<String>();
		tsCryptInfo.add(lbxCertificates, BorderLayout.CENTER);
		
		tsErrorInfo = new JPanel();
		//PageControl.addTab("Error Details", null, tsErrorInfo, null);
		tsErrorInfo.setLayout(new BorderLayout(0, 0));
		
		scrollPane = new JScrollPane();
		tsErrorInfo.add(scrollPane, BorderLayout.CENTER);
		
		mErr = new JTextArea();
		mErr.setText(" ");
		scrollPane.setViewportView(mErr);
		
		init();
	}

	private void init() {
		fCaption = "SMime Part";
	}

	protected void viewClick() {
		btnViewCert_Click();
	}
	
	private TElMessagePartHandlerSMime ph;
	private boolean fDecoderIsSigned;
	private boolean fDecoderIsCrypted;

	protected void Init(TElMessagePart messagePart, TagInfo tagInfo, TreeNodeInfo treeNodeItem, boolean bShow)
	{
		fTagInfo = tagInfo;
		fElMessagePart = messagePart;
		fNode = treeNodeItem;

		if ( (fElMessagePart == null) || (! bShow) )
			return;

		if (treeNodeItem == null)
			return;

		if ((messagePart.getMessagePartHandler() == null) || !(messagePart.getMessagePartHandler() instanceof TElMessagePartHandlerSMime))
		{																										  
			PageControl.setVisible(false);
			return;
		}

		ph = (TElMessagePartHandlerSMime)messagePart.getMessagePartHandler();
		PageControl.setVisible(true);

		InitData();
		if (ph.getIsError())
			PageControl.setSelectedComponent(tsErrorInfo);
		else if (fDecoderIsSigned)
			PageControl.setSelectedComponent(tsSignInfo);
		else if (fDecoderIsCrypted)
			PageControl.setSelectedComponent(tsCryptInfo);
		else
			PageControl.setSelectedComponent(tsSignInfo);

		if (PageControl.getSelectedComponent() != tsErrorInfo)
			PageControl_SelectedIndexChanged();

		PageControl.setVisible(true);
	}

	private boolean tsErrorInfoVisible = true;

	private JTabbedPane PageControl;

	private void InitData()
	{
		if ((ph != null) && ph.getIsError())
		{
			mErr.setText(ph.getErrorText());

			if (!tsErrorInfoVisible)
			{
				PageControl.addTab("Error Details", null, tsErrorInfo, null);
				tsErrorInfoVisible = true;
			}
		}
		else
		{
			if (tsErrorInfoVisible)
			{
				PageControl.removeTabAt(PageControl.getTabCount() - 1);
				tsErrorInfoVisible = false;
			}

			mErr.setText("");
		}

		if (ph != null) 
		{
			fDecoderIsSigned = ph.decoderIsSigned();
			fDecoderIsCrypted = ph.decoderIsCrypted();
		}
		else
		{
			fDecoderIsSigned = false;
			fDecoderIsCrypted = false;
		}
	}

	private void PageControl_SelectedIndexChanged()
	{
		if (PageControl.getSelectedComponent() == tsSignInfo)
		{
			if (fDecoderIsSigned && (ph.getDecoderSignCertStorage() != null) && (ph.getDecoderSignCertStorage().getCount() > 0))
			{
				SetCertificates(ph.getDecoderSignCertStorage());
				
				tsSignInfo.add(pBtns, BorderLayout.EAST);
				lbxCertificates.setVisible(true);
				pBtns.setVisible(true);
			}
			else
			{
				if (ph.getIsError())
					tsErrorInfo.setVisible(true);

				lbxCertificates.setVisible(false);
				pBtns.setVisible(false);
			}
		}
		else if (PageControl.getSelectedComponent() == tsCryptInfo)
		{
			if (fDecoderIsCrypted && (ph.getDecoderCryptCertStorage() != null) && (ph.getDecoderCryptCertStorage().getCount() > 0))
			{
				SetCertificates(ph.getDecoderCryptCertStorage());

				tsCryptInfo.add(pBtns, BorderLayout.EAST);

				lbxCertificates.setVisible(true);
				pBtns.setVisible(true);
			}
			else
			{
				if (ph.getIsError())
					tsErrorInfo.setVisible(true);

				lbxCertificates.setVisible(false);
				pBtns.setVisible(false);
			}
		}
	}

	public static String GetOIDValue(TElRelativeDistinguishedName RDN, byte[] S)
	{
		String t = "";
		int iCount = RDN.getCount();
		for (int i = 0; i < iCount; i++)
		{
			if (SBUtils.compareContent(RDN.getOID(i), S))
			{
				if (t != "")
					t += " / ";

				t += SBStrUtils.utf8ToStr(RDN.getValue(i));
			}
		}

		return t;
	}
	
	private void SetCertificates(TElCustomCertStorage CertStorage)
	{
		try
		{
			DefaultListModel<String> model = (DefaultListModel<String>)lbxCertificates.getModel();
			model.clear();

			String s;
			TElX509Certificate cer;
			for (int i = 0; i < CertStorage.getCount(); i++)
			{
				cer = CertStorage.getCertificate(i);
				s = GetOIDValue(cer.getSubjectRDN(), SBConstants.SB_CERT_OID_COMMON_NAME.Data) + " / " + 
					GetOIDValue(cer.getSubjectRDN(), SBConstants.SB_CERT_OID_EMAIL.Data);
				
				model.addElement(s);
			}
		}
		finally
		{
		}
	}

	private void btnViewCert_Click()
	{
		int idx = lbxCertificates.getSelectedIndex();
		if (idx < 0)
			return;

		MimeViewer_CertDetails CertInfo = new MimeViewer_CertDetails();
		try
		{
			TElX509Certificate cer;
			if (PageControl.getSelectedComponent() == tsCryptInfo)
				cer = ph.getDecoderCryptCertStorage().getCertificate(idx);
			else
				cer = ph.getDecoderSignCertStorage().getCertificate(idx);

			CertInfo.SetCertificate(cer);
			CertInfo.setVisible(true);
		}
		finally
		{
			CertInfo.dispose();
		}
	}
	
}
