import java.io.*;
import java.util.UUID;

import javax.swing.JOptionPane;
import javax.swing.JPasswordField;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.Base.Thread;
import SecureBlackbox.PGP.*;

public class PanelProcessProtectionDescriptor extends WizardPanelDescriptor {
    
    public static final String IDENTIFIER = "PROTECT_PROCESS_PANEL";
    
    TElPGPWriter pgpWriter;
    PanelProcessProtection panel;
    TElStringList temporaryFiles;
    	
    public PanelProcessProtectionDescriptor() {
        panel = new PanelProcessProtection();
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(panel);
        temporaryFiles = new TElStringList();
    }

    public Object getNextPanelDescriptor() {
        return FINISH;
    }
    
    public Object getBackPanelDescriptor() {
        return PanelSelectDestDescriptor.IDENTIFIER;
    }
        
    public void aboutToDisplayPanel() {
        panel.setProgressValue(0);
        panel.setProgressText("Protecting file...");

        getWizard().setNextFinishButtonEnabled(false);
        getWizard().setBackButtonEnabled(false);
    }
    
    public void displayingPanel() {
   		
		Thread t = new Thread(){
    		public void run() {
	   			try
	   	    	{
	   				ProtectFile(ProtectionProps.getSourcePath(), ProtectionProps.getDestPath());

			        panel.setProgressText("Data was successfully protected");
	   				
			        DestroyTempFiles();
	   				
			        getWizard().setNextFinishButtonEnabled(true);
			        getWizard().setBackButtonEnabled(true);
			    } catch (Exception e) {
			        
			        panel.setProgressValue(0);
			        showError(e);
			        
			        getWizard().setBackButtonEnabled(true);
			    }
    		}
		};
		
		t.start();
    }
    
	void showError(Exception e){
		panel.setProgressText("An Error Has Occurred. " + e.getMessage());
	}
    
	private void ProtectFile(final String srcFile, final String destFile) 
	{       
		pgpWriter = new TElPGPWriter();
		if (ProtectionProps.isMakeSFX())
		{
			/*JFileChooser chooser = new JFileChooser();
		
			if (chooser.showSaveDialog(panel) == JFileChooser.APPROVE_OPTION)
			{
		        File file = chooser.getSelectedFile();
                pgpWriter.SetStubSource(TSBPGPStubSource.ssFile);
                pgpWriter.SetStubName(file.getName());
                pgpWriter.SetSFXEnabled(true);
			}*/	
		} else {
            pgpWriter.setArmor(ProtectionProps.isArm());
            if (pgpWriter.getArmor())
            {
                pgpWriter.getArmorHeaders().clear();
                pgpWriter.getArmorHeaders().add("Version: EldoS OpenPGPBlackbox (Java edition)");
                pgpWriter.setArmorBoundary("PGP MESSAGE");
            }			
		}
		
		pgpWriter.setCompress(ProtectionProps.isCompress());
        pgpWriter.setCompressionAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_CM_ZLIB);
        pgpWriter.setCompressionLevel(9);           
		
        TElPGPKeyring pubKeyring = new TElPGPKeyring();
        
		boolean isUseConvEnc = ProtectionProps.isUseConvEnc();
		
        pubKeyring = ProtectionProps.getPubKeyring();
        pgpWriter.setEncryptingKeys(pubKeyring);
		
		if ((isUseConvEnc) && (pubKeyring.getPublicCount() > 0)) 
		{
			pgpWriter.setEncryptionType(TSBPGPEncryptionType.etBoth);
		} 
		else if ((isUseConvEnc) && (pubKeyring.getPublicCount() == 0)) 
		{
			pgpWriter.setEncryptionType(TSBPGPEncryptionType.etPassphrase);
		} 
		else 
		{
			pgpWriter.setEncryptionType(TSBPGPEncryptionType.etPublicKey);
		}
		
		boolean isTextInput = ProtectionProps.isTextInput();
		File info = new File(srcFile);
		pgpWriter.setFilename(info.getName());
		pgpWriter.setInputIsText(isTextInput);
		pgpWriter.getPassphrases().clear();
		pgpWriter.getPassphrases().add(ProtectionProps.getPassphrase());
		switch(ProtectionProps.getProtLevel()) {
			case 0 : 
				pgpWriter.setProtection(TSBPGPProtectionType.ptLow);
				break;
			case 1 :
				pgpWriter.setProtection(TSBPGPProtectionType.ptNormal);
				break;
			default:
				pgpWriter.setProtection(TSBPGPProtectionType.ptHigh);
				break;
		}
		pgpWriter.setSignBufferingMethod(TSBPGPSignBufferingMethod.sbmRewind);
		switch (ProtectionProps.getEncryptionAlg()) 
		{
			case 0 : 
				pgpWriter.setSymmetricKeyAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_SK_CAST5);
				break;
			case 1 :
				pgpWriter.setSymmetricKeyAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_SK_3DES);
				break;
			case 2 :
				pgpWriter.setSymmetricKeyAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_SK_AES128);
				break;
			default :
				pgpWriter.setSymmetricKeyAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_SK_AES256);
				break;
		}

		boolean isSign = ProtectionProps.isSign();
		if (isSign)
		{
	        TElPGPKeyring secKeyring = ProtectionProps.getSecKeyring();
			pgpWriter.setSigningKeys(secKeyring);

			switch (ProtectionProps.getHashAlgorithm())
			{
				case 0 :
					pgpWriter.setHashAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_MD_MD5);
					break;
				case 1 :
					pgpWriter.setHashAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_MD_SHA1);
					break;
				case 2 :
					pgpWriter.setHashAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_MD_RIPEMD160);
					break;
				case 3 :
					pgpWriter.setHashAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_MD_SHA256);
					break;
				case 4 :
					pgpWriter.setHashAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_MD_SHA384);
					break;
				case 5 :
					pgpWriter.setHashAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_MD_SHA512);
					break;
				default :
					pgpWriter.setHashAlgorithm(SBPGPConstants.SB_PGP_ALGORITHM_MD_SHA1);
					break;
			}
		}  	
		   	
		pgpWriter.setOnProgress(new TSBPGPProgressEvent(progressCallback));
		pgpWriter.setOnKeyPassphrase(new TSBPGPKeyPassphraseEvent(keyPassCallback));
		
		pgpWriter.setUseNewFeatures(ProtectionProps.useNewFeatures());
		pgpWriter.setUseOldPackets(ProtectionProps.useOldPackets());

		TElFileStream inF = null;
		TElFileStream outF = null;
		boolean isEncrypt = ProtectionProps.isEncrypt();
		try 
		{
			inF = new TElFileStream(srcFile, "r", true);
			outF = new TElFileStream(destFile, "rw", true);
			if ((!isEncrypt) && (isSign) && (isTextInput))
			{
				pgpWriter.clearTextSign(inF, outF, 0);
			} 
			else if (isEncrypt && isSign) 
			{
				pgpWriter.encryptAndSign(inF, outF, 0);
			} 
			else if (isEncrypt && (!isSign))
			{
				pgpWriter.encrypt(inF, outF, 0);
			} 
			else 
			{
				pgpWriter.sign(inF, outF, false, 0);
			}
		} 
		catch (Exception e) {
			showError(e);
		}			
		finally 
		{
			if (inF != null) inF.Destroy();
			if (outF != null) outF.Destroy();
		}
	}	
	
	void DestroyTempFiles() {
        if (temporaryFiles.size() > 0)
        {
            for (int i = 0; i < temporaryFiles.size(); i++)
                new File((String) temporaryFiles.get(i)).delete();
            temporaryFiles.clear();
        }		
	}
	
    // event's
	   
	TSBPGPProgressEvent.Callback progressCallback = new TSBPGPProgressEvent.Callback() {
		public void tsbpgpProgressEventCallback(TObject sender, long passed, long total, TSBBoolean cancel) {
			if (total != 0) {
				int progress = (int) (passed * 100 / total); 
				panel.setProgressValue(progress);
			}
			cancel.value = false;
		}
	}; 
	
    TSBPGPKeyPassphraseEvent.Callback keyPassCallback = new TSBPGPKeyPassphraseEvent.Callback() {
		public void tsbpgpKeyPassphraseEventCallback(TObject sender, TElPGPCustomSecretKey key, TSBString pass, TSBBoolean reject) {
			String p = requestKeyPassphrase(key);
			if (p.length() == 0) {
				 reject.value = true;
			} else {
			pass.value = p;
			reject.value =  false;
			}
		}

		private String requestKeyPassphrase(TElPGPCustomSecretKey key) {
			JPasswordField jpf = new JPasswordField();
			int result = JOptionPane.showConfirmDialog(null, jpf, "Passphrase is needed for secret key", JOptionPane.OK_CANCEL_OPTION);
			if (result == 0)
				return new String(jpf.getPassword());
			return "";
		}
	};
    
	TSBPGPTemporaryStreamEvent.Callback tmpCallback = new TSBPGPTemporaryStreamEvent.Callback() {
		public void tsbpgpTemporaryStreamEventCallback(TObject sender, TElPGPCreateStreamEventParams params) {
			String fileName = System.getProperty("java.io.tmpdir") + "/" + UUID.randomUUID().toString();
			try {
				params.Stream = new TElFileStream(fileName, "rw", true);
			} catch (Exception e) {
				showError(e);
			}
			temporaryFiles.add(fileName);
			params.FreeOnFinish = true;
		}
	};	
}