package com.eldos.idp;

import SecureBlackbox.SAMLIDP.TElSAMLServiceProviderInfo;
import SecureBlackbox.XML.TElXMLDOMDocument;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LoginServlet extends HttpServlet {

    private Map<String, String> credentials = new HashMap<>();

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        String userCountStr = config.getInitParameter("user_count");
        int userCount = Integer.parseInt(userCountStr);

        String login, pass;
        for (int i = 1; i <= userCount; i++) {
            login = config.getInitParameter("user" + i + "_login");
            pass = config.getInitParameter("user" + i + "_password");

            if (login == null || pass == null)
                break;
            else
                credentials.put(login, pass);
        }

        if (credentials.isEmpty())
            throw new RuntimeException("No login credentials configured!");
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String login = req.getParameter("login");
        String pass = req.getParameter("password");
        String storedPass = credentials.get(login);

        if (storedPass != null && storedPass.equals(pass)) {
            HttpSession session = req.getSession(false);
            if (session == null) {
                resp.setStatus(400); // session should exist on this step
                return;
            }

            session.setAttribute("loggedIn", true); // mark as logged in
            session.setAttribute("login", login); // mark as logged in

            // extract parameters
            final TElXMLDOMDocument doc = (TElXMLDOMDocument) session.getAttribute("requestBody");
            final String state = (String) session.getAttribute("requestState");
            final List<TElSAMLServiceProviderInfo> spList = (List<TElSAMLServiceProviderInfo>) session.getAttribute("spList");
            final String prefferedSSOResponseBinding = (String) session.getAttribute("prefferedSSOResponseBinding");
            final String idpIssuer = (String) session.getAttribute("idpIssuer");
            final String signCertPath = (String) session.getAttribute("signCertPath");
            final boolean signResponses = (session.getAttribute("signResponses") != null) ?
                    (Boolean) session.getAttribute("signResponses") : false;

            // process SAML request
            Utils.processRequest(req, resp, doc, login, state, spList, prefferedSSOResponseBinding,
                    idpIssuer, signCertPath, signResponses);

            session.setAttribute("requestBody", null);
            session.setAttribute("requestState", null);
            session.setAttribute("spList", null);
            session.setAttribute("prefferedSSOResponseBinding", null);
            session.setAttribute("idpIssuer", null);
            session.setAttribute("signCertPath", null);
            session.setAttribute("signResponses", null);
        } else {
            RequestDispatcher view = req.getRequestDispatcher("/WEB-INF/login.html");
            view.forward(req, resp);
        }
    }
}
