import java.util.*;

import javax.swing.table.AbstractTableModel;

import SecureBlackbox.SSHClient.*;

public class ConTableModel extends AbstractTableModel {
	private static final long serialVersionUID = 1L;
	public static final int ICON_INDEX = 0;
	public static final int HOST_INDEX = 1;
	public static final int R_INDEX = 2;
	public static final int S_INDEX = 3;
	public static final int RC_INDEX = 4;
	public static final int SC_INDEX = 5;
	public static final int SS_INDEX = 6;
	public static final int CS_INDEX = 7;
	protected Vector<ConDataRow> dataVector = new Vector<ConDataRow>();
	
    protected String[] columnNames = {
    		"",
    		"Host", 
    		"Incoming (socket)",
    		"Outgoing (socket)",
    		"Incoming (channel)",
    		"Outgoing (channel)",
    		"Socket state",
    		"Channel state"
    };

	public int getColumnCount() {
		return columnNames.length;
	}

 	public String getColumnName(int column) {
        return columnNames[column];
    }

 	public Class<?> getColumnClass(int column) {
       	return String.class;
    }

	public int getRowCount() {
		return dataVector.size();
	}

    public void setValueAt(Object value, int row, int column) {
      	 ConDataRow record = (ConDataRow)dataVector.get(row);
           switch (column) {
           	   case ICON_INDEX:
           		   record.Status = (String)value;
           		   break;
               case HOST_INDEX:
                  record.Host = (String)value;
                  break;
               case R_INDEX:
                   record.Received = (String)value;
                   break;
               case S_INDEX:
                   record.Sent = (String)value;
                   break;
               case RC_INDEX:
                   record.ReceivedCh = (String)value;
                   break;
               case SC_INDEX:
                   record.SentCh = (String)value;
                   break;
               case SS_INDEX:
                   record.SocketState = (String)value;
                   break;
               case CS_INDEX:
                   record.ChannelState = (String)value;
                   break;                   
               default:
                  System.out.println("invalid index");
           }
           fireTableCellUpdated(row, column);
       } 	
	
	public Object getValueAt(int row, int col) {
        ConDataRow record = (ConDataRow)dataVector.get(row);
        switch (col) {
        	case ICON_INDEX:
        		return record.Status;
	        case HOST_INDEX:
	        	return record.Host;
            case R_INDEX:
            	return record.Received;
            case S_INDEX:
            	return record.Sent;
            case RC_INDEX:
            	return record.ReceivedCh;
            case SC_INDEX:
            	return record.SentCh;    
            case SS_INDEX:
                return record.SocketState;
            case CS_INDEX:
                return record.ChannelState;   	
            default:
            	return new Object();
        }
	}

	public void addRow(TElSSHForwardedConnection c) {
		String host = "N/A";
        if (c.getSocket() != null)
			c.getSocket().getAddress();
        
		String received = c.getReceivedFromSocket() + "";
		String sent = c.getSentToSocket() + "";

		String receivedCh = c.getReceivedFromChannel() + "";
		String sentCh = c.getSentToChannel() + "";
		
		String socketState = getCurrentSocketState(c);
		String chState = getCurrentChannelState(c);
		String status = getStatus(c);
		Object tag = c;
		dataVector.addElement(new ConDataRow(status, host, received, sent, receivedCh, sentCh, socketState, chState, tag));
		fireTableDataChanged();
	}

	private String getStatus(TElSSHForwardedConnection c) {
        String s = "";
		if ((c.getChannelState() == TSBSSHForwardingChannelState.fcsActive) &&
                (c.getSocketState() == TSBSSHForwardingSocketState.fssActive))
            {
                s = "active";
            }
            else if ((c.getChannelState() == TSBSSHForwardingChannelState.fcsEstablishing) ||
                (c.getSocketState() == TSBSSHForwardingSocketState.fssEstablishing))
            {
                s = "establ";
            }
            else
            {
                s = "closed";
            }
        return s;
	}

	private String getCurrentChannelState(TElSSHForwardedConnection c) {
		String s = "";
		
	    if (c.getChannelState() == TSBSSHForwardingChannelState.fcsEstablishing)
	        s = "Establishing";

	    if (c.getChannelState() == TSBSSHForwardingChannelState.fcsActive)
	        s = "Active";

	    if (c.getChannelState() == TSBSSHForwardingChannelState.fcsClosing)
	        s = "Closing";

	    if (c.getChannelState() == TSBSSHForwardingChannelState.fcsClosed)
	        s = "Closed";
		
	    return s;
	}

	private String getCurrentSocketState(TElSSHForwardedConnection c) {
		String s = "";
		if (c.getSocketState() == TSBSSHForwardingSocketState.fssEstablishing)
            s = "Establishing";
		if (c.getSocketState() == TSBSSHForwardingSocketState.fssActive)
	        s = "Active";
        if (c.getSocketState() == TSBSSHForwardingSocketState.fssClosing)
        	s = "Closing";
        if (c.getSocketState() == TSBSSHForwardingSocketState.fssClosed)
        	s = "Closed";
		return s;
	}

	public void change(TElSSHForwardedConnection c) {
		remove(c);
		addRow(c);
	}

	public void remove(TElSSHForwardedConnection c) {
		int index = -1;
		for (int i = 0; i < dataVector.size(); i++) 
		{
			if (dataVector.get(i).Tag.equals(c)) 
			{
				index = i;
				break;
			}
		}
		
		if (index != -1)
			dataVector.remove(index);
		
		fireTableDataChanged();
	}
}
