import SecureBlackbox.Base.*;
import SecureBlackbox.Base.Thread;

public abstract class TcpListener {
	private int m_port;
	private boolean destroy;
	private String m_host;
	TElSocket clientSocket;
	abstract void connected(TElSocket socket);
	abstract void started(TElSocket socket);
	
	TElSocket m_socket;

	public TcpListener() { super(); }
	public TcpListener(int port) { m_port = port; }
	public TcpListener(String host, int port) { m_host = host; m_port = port; }
	
	void accept() {
		destroy = false;		
		try {
			processAccept();
		} catch (InterruptedException e) {
			Logger.Log(e.getMessage());
		}
	}
	
	void endAccept()
	{
		//destroy = true;
	}
	
	private void processAccept() throws InterruptedException {
		synchronized (this) {
			try
			{
				while (!destroy) {
					TSBObject obj = new TSBObject();
					m_socket.accept(1, obj);
	
					clientSocket = (TElSocket)obj.value;
					
					if (clientSocket == null)
					{
						Thread.sleep(1000);
						continue;
					}
					
					break;
				}
	
				endAccept();
				
				if (clientSocket != null)
				{
					Logger.Log("Client accepted. Host: " + 
							clientSocket.getRemoteAddress() + " port: " +
							clientSocket.getBoundPort());
		
					connected(clientSocket);
					
					accept();
				}
			}
			catch (InterruptedException e) { 				
			}				
			catch(Exception e) {
				e.printStackTrace();
				Logger.Log(e.getMessage());
			}
		}
	}

	public TElSocket start() {
		try 
		{
			m_socket = new TElSocket();

			m_socket.setAddress(m_host);
			m_socket.setListenPort(m_port);
	  	
		  	if (m_socket.bind() == 0) {
		  		if (m_socket.listen() == 0) {
		  			started(m_socket);
		  		} else {
		  			Logger.Log("Failed to bind socket to the port");
		  		}
		  	}
		  	else {
		  		Logger.Log("Failed to bind socket to the port");
		  	}
		} 
		catch(Exception ex)
		{
			reset();
			Logger.Log("Start listenining failed");
			Logger.Log("Start listenining failed - " + ex.getMessage());			
		}
		return m_socket;
	}

	public void stop() {
		try {
			destroy = true;
			reset();
			m_socket.close(false); 
		} catch (Exception ex) {
			Logger.Log("Stop failed - " + ex.getMessage());
		}
	}
	
	private void reset() {
		if (clientSocket != null) {
			clientSocket.close(false);
			clientSocket = null;
		}
	}
		
	public boolean pending() {
		return m_socket.canAccept(1000);
	}
	
	public TElSocket getSocket() {
		return m_socket;
	}
}