import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;

import SecureBlackbox.SSHCommon.SBSSHConstants;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.ButtonGroup;


public class UserDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	JTextField edUserName;
	JCheckBox cbPassword;
	JButton btnSetPassword;
	JCheckBox cbPublicKey;
	JButton btnSetPublicKey;
	JCheckBox cbKeyboard;
	JRadioButton rbAny;
	JRadioButton rbAll;

	String Password = "";
	String PublicKey = "";
	private boolean isOk;
	private final ButtonGroup buttonGroup = new ButtonGroup();
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			UserDialog dialog = new UserDialog(new UserInfo());
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 * @param user 
	 */
	public UserDialog(UserInfo user) {
		setTitle("User settings");
		setBounds(100, 100, 450, 269);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblUserName = new JLabel("User name:");
		lblUserName.setBounds(10, 11, 91, 14);
		contentPanel.add(lblUserName);
		
		edUserName = new JTextField();
		edUserName.setBounds(111, 8, 321, 20);
		contentPanel.add(edUserName);
		edUserName.setColumns(10);
		
		JPanel panel = new JPanel();
		panel.setBorder(new TitledBorder(null, "Allowed authentications:", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		panel.setBounds(10, 36, 422, 156);
		contentPanel.add(panel);
		panel.setLayout(null);
		
		cbPassword = new JCheckBox("Allow password authentication");
		cbPassword.setBounds(6, 18, 263, 23);
		panel.add(cbPassword);
		
		btnSetPassword = new JButton("Set password");
		btnSetPassword.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setPassword();
			}
		});
		btnSetPassword.setBounds(281, 18, 131, 23);
		panel.add(btnSetPassword);
		
		cbPublicKey = new JCheckBox("Allow public-key authentication");
		cbPublicKey.setBounds(6, 44, 263, 23);
		panel.add(cbPublicKey);
		
		btnSetPublicKey = new JButton("Set public key");
		btnSetPublicKey.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setPublicKey();
			}
		});
		btnSetPublicKey.setBounds(281, 48, 131, 23);
		panel.add(btnSetPublicKey);
		
		cbKeyboard = new JCheckBox("Allow keyboard-interactive authentication");
		cbKeyboard.setBounds(6, 70, 271, 23);
		panel.add(cbKeyboard);
		
		rbAny = new JRadioButton("The user must authenticate using ANY of the above methods");
		buttonGroup.add(rbAny);
		rbAny.setBounds(6, 96, 406, 23);
		panel.add(rbAny);
		
		rbAll = new JRadioButton("The user must authenticate using ALL of the above methods");
		buttonGroup.add(rbAll);
		rbAll.setBounds(6, 122, 406, 23);
		panel.add(rbAll);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = false;
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		initDialog(user);
	}
	
	protected void setPublicKey() {
        PublicKey = KeyDialog.GetKey();
    }

	protected void setPassword() {
		Password = PasswordDialog.GetPassword();
	}

	private void initDialog(UserInfo user)
	{
		edUserName.setText(user.getName());
		cbPassword.setSelected((user.getAuthTypes() & SBSSHConstants.SSH_AUTH_TYPE_PASSWORD) != 0);
		cbPublicKey.setSelected((user.getAuthTypes() & SBSSHConstants.SSH_AUTH_TYPE_PUBLICKEY) != 0);
		cbKeyboard.setSelected((user.getAuthTypes() & SBSSHConstants.SSH_AUTH_TYPE_KEYBOARD) != 0);
		rbAll.setSelected(user.getAuthAll());
		rbAny.setSelected(!rbAll.isSelected());
		Password = "";
		PublicKey = "";
	}
	
	public void showMessage(String caption, String msg) {
		JOptionPane.showMessageDialog(null, msg, caption, JOptionPane.INFORMATION_MESSAGE);
	}	

	public boolean isOK() {
		return isOk;
	}
	
	public static boolean EditUserParameters(UserInfo user) {
		UserDialog frm = new UserDialog(user);
		
		frm.setModal(true);
		frm.setVisible(true);
		if (frm.isOK())
		{
			if (frm.Password.length() > 0) 
			{
				user.SetPassword(frm.Password);
			}
			if (frm.PublicKey.length() > 0) 
			{
				if (!user.SetPublicKey(frm.PublicKey)) 
				{
					frm.showMessage("Error", "Failed to set private key: bad private key");
				}
			}
			user.setAuthAll(frm.rbAll.isSelected());
			user.setName(frm.edUserName.getText());
			int Auth = 0;
			if (frm.cbKeyboard.isSelected()) 
				Auth |= SBSSHConstants.SSH_AUTH_TYPE_KEYBOARD;
			if (frm.cbPassword.isSelected()) 
				Auth |= SBSSHConstants.SSH_AUTH_TYPE_PASSWORD;
			if (frm.cbPublicKey.isSelected()) 
				Auth |= SBSSHConstants.SSH_AUTH_TYPE_PUBLICKEY;
			user.setAuthTypes(Auth);
			frm.dispose();
			return true;
		}
		else {
			frm.dispose(); 
			return false;
		}
	}
}
