import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import SecureBlackbox.XML.*;
import SecureBlackbox.XMLSecurity.*;

public class SignaturesTableModel extends AbstractTableModel {
	
	private static final long serialVersionUID = 1L;

	private static final String[] columnNames = {
    		"Handler Name",
    		"Path", 
    		"Options",
    		"XAdES Info",
    };
	
	private List<SignatureRow> data = new ArrayList<SignatureRow>();

	public int getRowCount() {
		return data.size();
	}
	
	public String getColumnName(int column) {
        return columnNames[column];
    }

	public int getColumnCount() {
		return columnNames.length;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		SignatureRow row = data.get(rowIndex);
		String res = null;
		
		switch (columnIndex) {
		case 0:
			res = row.getHandlerName();
			break;
		case 1:
			res = row.getPath();
			break;
		case 2:
			res = row.getOptions();
			break;
		case 3:
			res = row.getXadesInfo();
			break;
		}
		
		return res;
	}
	
	public void clear() {
		data.clear();
		fireTableDataChanged();
	}
	
	public void add(TElXMLSOAPCustomSignatureHandler handler) {
		final String handlerName = handler.getClass().getSimpleName();
		final String path = getElementNicePath(handler.getXMLElement());
		
		StringBuilder s = new StringBuilder();
		
		if (handler instanceof TElXMLSOAPBaseSignatureHandler) {
			String t = ((TElXMLSOAPBaseSignatureHandler)handler).getKeyName();
			if (t.length() > 0)
				s.append("Key Name: ").append(t).append(", ");
		}
		
		if (handler instanceof TElXMLSOAPSignatureHandler) {
			String t = ((TElXMLSOAPSignatureHandler)handler).getID();
			if (t.length() > 0)
				s.append("ID: ").append(t).append(", ");
			
			t = ((TElXMLSOAPSignatureHandler)handler).getActor();
			if (t.length() > 0)
				s.append("Actor: ").append(t).append(", ");
			
			t = ((TElXMLSOAPSignatureHandler)handler).getMustUnderstand();
			if (t.length() > 0)
				s.append("MustUnderstand: ").append(t).append(", ");
		}
		
		if (handler instanceof TElXMLSOAPBaseSignatureHandler)
			s.append("Certificates: ").append(((TElXMLSOAPBaseSignatureHandler)handler).getCertificates().getCount());
		
		final String options = s.toString();
		
		s.setLength(0);
		if (handler instanceof TElXMLSOAPBaseSignatureHandler) {
			if (((TElXMLSOAPBaseSignatureHandler)handler).getXAdESProcessor() instanceof TElXAdESVerifier) {
				TElXAdESVerifier verifier = (TElXAdESVerifier)((TElXMLSOAPBaseSignatureHandler)handler).getXAdESProcessor();
				if (verifier.getIsEnabled())
					s.append(SBXMLAdES.xAdESFormToString(verifier.getXAdESForm()))
					 .append(", ")
					 .append(SBXMLAdES.xAdESVersionToString(verifier.getXAdESVersion()));
			}
		}
		
		final String xadesInfo = (s.length() > 0) ? s.toString() : "none";
			
		// add
		data.add(new SignatureRow(handlerName, path, options, xadesInfo));
		fireTableDataChanged();
	}
	
	public void remove(int index) {
		data.remove(index);
		fireTableDataChanged();
	}
	
	private String getElementNicePath(TElXMLDOMElement el) {
		if (el == null)
			return "";
		
		StringBuilder sb = new StringBuilder();
		
		if (el.getParentNode() != null) {
			if (el.getParentNode() instanceof TElXMLDOMElement)
				sb.append(getElementNicePath((TElXMLDOMElement)el.getParentNode()) + "/");
			else if (el.getParentNode() instanceof TElXMLDOMDocument)
				sb.append("/");
				
			int index = 0;
			boolean nonUnique = false, found = false;
			TElXMLDOMNode node = el.getParentNode().getFirstChild();
			
			while (node != null) {
				if ((node.getNodeType() == SBXMLCore.ntElement) && (node.getNodeName().equals(el.getNodeName()))) {
					if (node == el)
						found = true;
					else {
						nonUnique = true;
						if ( ! found)
							index++;
					}
				}
				
				node = node.getNextSibling();
			}
			
			sb.append(el.getNodeName());
			if (nonUnique)
				sb.append(String.format("[%d]", index));
			
			return sb.toString();
		} else
			return el.getNodeName();
	}
}
