

import SecureBlackbox.Base.*;
import SecureBlackbox.ZIP.*;
import org.freepascal.rtl.TObject;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

public class Main {

	private JFrame frmTarArchivier;
	private JTable table;

	String currentPath;
	String extractToPath;
	TElTarWriter elTarWriter;

	JMenuItem mnCloseArchive;
	JMenuItem mntmSaveArchive;
	JMenuItem mntmSaveArchiveAs;
	private JLabel lblCurrentPath;
	String newArchiveName;
    String saveArchiveName;
	String openedArchiveName;
    TSBTarCompressMethod oldCM;

	ProgressDialog progressForm;	
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {

		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main window = new Main();
					window.frmTarArchivier.setVisible(true);
				} catch (Exception e) {
					showErrorMessage("Error", e.getMessage());
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public Main() {
		initialize();
		initTarWriterEvents();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
        #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SBUtils.setLicenseKey("<put the key here>");
		
 		SBChSConvCharsets.initialize();

        elTarWriter = new TElTarWriter();

        frmTarArchivier = new JFrame();
        frmTarArchivier.setTitle("TAR Archivier");
        frmTarArchivier.setBounds(100, 100, 450, 300);
        frmTarArchivier.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		JMenuBar menuBar = new JMenuBar();
        frmTarArchivier.setJMenuBar(menuBar);
		
		JMenu mnFile = new JMenu("File");
		menuBar.add(mnFile);
		
		JMenuItem mntmOpenArchive = new JMenuItem("Open archive...");
		mntmOpenArchive.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openArchive();
			}
		});
		mnFile.add(mntmOpenArchive);
		
		JMenuItem mntmNewArchive = new JMenuItem("New archive...");
		mntmNewArchive.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				newArchive();
			}
		});
		mnFile.add(mntmNewArchive);
		
		mnCloseArchive = new JMenuItem("Close archive");
		mnCloseArchive.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				closeArchive();
			}
		});
		mnCloseArchive.setEnabled(false);
		mnFile.add(mnCloseArchive);
		
		mntmSaveArchive = new JMenuItem("Save archive");
		mntmSaveArchive.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveArchive();
			}
		});
		mntmSaveArchive.setEnabled(false);
		mnFile.add(mntmSaveArchive);
		

		mntmSaveArchiveAs = new JMenuItem("Save archive as...");
		mntmSaveArchiveAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveArchiveAs();
			}
		});
		mntmSaveArchiveAs.setEnabled(false);
		mnFile.add(mntmSaveArchiveAs);
		
		JMenuItem mntmExit = new JMenuItem("Exit");
		mntmExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
                frmTarArchivier.dispose();
			}
		});
		mnFile.add(mntmExit);
		
		table = new JTable();
		JScrollPane scrollPane = new JScrollPane(table);
        frmTarArchivier.getContentPane().add(scrollPane, BorderLayout.CENTER);
		
		lblCurrentPath = new JLabel(" ");
        frmTarArchivier.getContentPane().add(lblCurrentPath, BorderLayout.SOUTH);
		
		initPopup();
		initDoubleClick();
	}

	private void initDoubleClick() {
		table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e){
			      if (e.getClickCount() == 2){
			    	  openItem();
			      }
			}
		});
	}

	private void openItem() {
		if (table.getSelectedRowCount() > 0){
			TElTarArchiveDirectoryEntry entry = ((KeyTableModel)table.getModel()).getFirstSelectedItem(table.getSelectedRow());
			
			if (entry == null)
				currentPath = SBStrUtils.pathCutLastComponent(currentPath);
			else if (entry.getIsDirectory())
				currentPath = entry.getPath();
			else
				return;

			refreshFilesList();			
		}
	}
	
	private void initPopup() {
        JPopupMenu popup = new JPopupMenu();  
        JMenuItem popupExtract = new JMenuItem("Extract");  
        JMenuItem popupExtractTo = new JMenuItem("Extract to...");
        JMenuItem popupAddFiles = new JMenuItem("Add files...");
        JMenuItem popupAddDirectory = new JMenuItem("Add directory...");
        JMenuItem popupDelete = new JMenuItem("Delete");
        
        popupExtract.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				extractSelectedFiles(SBStrUtils.pathCutLastComponent(openedArchiveName));
			}
		});

        popupExtractTo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String directory = selectDirectory();
				extractSelectedFiles(directory);
			}
		});
        
        popupAddFiles.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addFilesToArchive();
			}
		});
        
        popupAddDirectory.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String directory = selectDirectory();
				TElTarArchiveDirectoryEntry entry;
				entry = elTarWriter.getDirectory().entryWithPath(currentPath);
				if (entry == null)
					entry = elTarWriter.getDirectory();
                elTarWriter.add(entry, directory, 0, null);
				refreshFilesList();				
			}
		});

        popupDelete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((KeyTableModel)table.getModel()).removeSelectedItems(table.getSelectedRows());
			}
		});
        
        popup.add(popupExtract);  
        popup.add(popupExtractTo);  
        popup.add(popupAddFiles);
        popup.add(popupAddDirectory);
        popup.add(popupDelete);
        
        MouseListener listener=new TablePopupListener(popup, table);  
        table.addMouseListener(listener);  
	}

	private void initTarWriterEvents() {
		elTarWriter.setOnUserActionNeeded(new TSBTarUserActionNeededEvent(actionClbk));
        elTarWriter.setOnCompressionFinished(new TSBTarCompressionFinishedEvent(finishClbk));
        elTarWriter.setOnCompressionStart(new TSBTarCompressionStartEvent(compressionStartClbk));
        elTarWriter.setOnExtractionFinished(new TSBTarExtractionFinishedEvent(extrClbk));
        elTarWriter.setOnProgress(new TSBTarProgressEvent(progressClbk));
        elTarWriter.setOnExtractionStart(new TSBTarExtractionStartEvent(extrStartClbk));
        elTarWriter.setOnUnCompressionTarFromGZipStart(new TSBTarUnCompressionTarFromGZipStartEvent(unCompressionGZipStartClbk));
        elTarWriter.setOnUnCompressionTarFromGZipFinished(new TSBTarUnCompressionTarFromGZipFinishedEvent(unCompressionGZipFinishClbk));
        elTarWriter.setOnUnCompressionTarFromBZip2Start(new TSBTarUnCompressionTarFromBZip2StartEvent(unCompressionBZip2StartClbk));
        elTarWriter.setOnUnCompressionTarFromBZip2Finished(new TSBTarUnCompressionTarFromBZip2FinishedEvent(unCompressionBZip2FinishClbk));
        elTarWriter.setOnCompressionTarToGZipStart(new TSBTarCompressionTarToGZipStartEvent(compressionGZipStartClbk));
        elTarWriter.setOnCompressionTarToGZipFinished(new TSBTarCompressionTarToGZipFinishedEvent(compressionGZipFinishClbk));
        elTarWriter.setOnCompressionTarToBZip2Start(new TSBTarCompressionTarToBZip2StartEvent(compressionBZip2StartClbk));
        elTarWriter.setOnCompressionTarToBZip2Finished(new TSBTarCompressionTarToBZip2FinishedEvent(compressionBZip2FinishClbk));
    }

	private void addFilesToArchive() {
		File[] files = getFileNames();
		for (int i = 0; i < files.length; i++)
            elTarWriter.add(elTarWriter.getDirectory(), files[i].getPath(), 0, null);
		refreshFilesList();
	}
	
	private String selectDirectory() {
		JFileChooser fc = new JFileChooser(); 
	    fc.setCurrentDirectory(new java.io.File("."));
	    fc.setDialogTitle("Select directory to extract files to");
	    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    fc.setAcceptAllFileFilterUsed(false);

	    if (fc.showOpenDialog(frmTarArchivier) == JFileChooser.APPROVE_OPTION)
	    	return fc.getSelectedFile().getPath();
	    
	    return "";
	}

	private File[] getFileNames() {
		JFileChooser chooser = new JFileChooser();
		chooser.setDialogTitle("Add files to archive...");
		chooser.setMultiSelectionEnabled(true);

		chooser.showOpenDialog(frmTarArchivier);

		return chooser.getSelectedFiles();			
	}

	String getFileName(){
		JFileChooser fc = new JFileChooser();

	    int returnVal = fc.showOpenDialog(frmTarArchivier);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	} 

	String getFileSaveAsName(){
		JFileChooser fc = new JFileChooser();

	    int returnVal = fc.showSaveDialog(frmTarArchivier);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	private void openArchive() {
		if (elTarWriter.getOpened())
            elTarWriter.close();

		String fileName = getFileName();
		if (fileName.length() > 0)
		{
			openedArchiveName = fileName;

            java.lang.Thread t = new java.lang.Thread(new Runnable() {
                public void run() {
                    try
                    {
                        elTarWriter.setCompressionMethod(TSBTarCompressMethod.cmNone);
                        if (openedArchiveName.indexOf(".tar.gz") > 0)
                            elTarWriter.setCompressionMethod(TSBTarCompressMethod.cmGZip);
                        if (openedArchiveName.indexOf(".tar.bz2") > 0)
                            elTarWriter.setCompressionMethod(TSBTarCompressMethod.cmBZip2);

                        elTarWriter.open(openedArchiveName, false);
                    }
                    catch (Exception ex)
                    {
                        showErrorMessage("Error opening archive", ex.getMessage());
                    }

                    if (elTarWriter.getOpened())
                    {
                        currentPath = "";
                        mnCloseArchive.setEnabled(true);
                        mntmSaveArchive.setEnabled(true);
                        mntmSaveArchiveAs.setEnabled(true);

                        refreshFilesList();
                    }
                    else
                    {
                        mnCloseArchive.setEnabled(false);
                    }
                }
            });

            t.start();
		}
	}

	static void showErrorMessage(String caption, String msg){
		JOptionPane.showMessageDialog(null, msg, caption, JOptionPane.ERROR_MESSAGE);
	}
	
	static void showMessage(String caption, String msg) {
		JOptionPane.showMessageDialog(null, msg, caption, JOptionPane.INFORMATION_MESSAGE);
	}	
	
	private void closeArchive() {
		if (elTarWriter.getOpened())
		{
            elTarWriter.close();
			clearTable();
			mnCloseArchive.setEnabled(false);
            mntmSaveArchive.setEnabled(false);
            mntmSaveArchiveAs.setEnabled(false);
		}
	}

	private void refreshFilesList()
	{
		if (!elTarWriter.getOpened())
			return;
	
		clearTable();

		table.setModel(new KeyTableModel(elTarWriter, currentPath));
		
		lblCurrentPath.setText(currentPath);
	}

	private void clearTable() {
		Object model = table.getModel();
		if (model instanceof KeyTableModel)
		{
			((KeyTableModel) model).dataVector.removeAllElements();
			((KeyTableModel) model).fireTableDataChanged();
		}
		else
			((DefaultTableModel) model).setNumRows(0);
	}

	private void createNewArchive(ArchiveOptionsDialog src) {
        newArchiveName = src.getNewArchiveFullPath();

        elTarWriter.createArchive();

        elTarWriter.setCompressionMethod(src.getCompressionAlg());
        elTarWriter.setCompressionLevel(src.getCompressionLevel());

		mnCloseArchive.setEnabled(true);
		mntmSaveArchive.setEnabled(true);
		mntmSaveArchiveAs.setEnabled(true);
		currentPath = "";

		addFilesToArchive();

	}

	private void newArchive() {
		closeArchive();
		
		ArchiveOptionsDialog dlg = new ArchiveOptionsDialog();
		dlg.setModal(true);
		dlg.setVisible(true);
		if (dlg.getIsCreateArch())
			createNewArchive(dlg);
	}

	private void saveArchiveAs() {
		saveArchiveName = getFileSaveAsName();

        if (saveArchiveName != "") {
            oldCM = elTarWriter.getCompressionMethod();

            elTarWriter.setCompressionMethod(TSBTarCompressMethod.cmNone);
            if (saveArchiveName.indexOf(".tar.gz") > 0)
                elTarWriter.setCompressionMethod(TSBTarCompressMethod.cmGZip);
            if (saveArchiveName.indexOf(".tar.bz2") > 0)
                elTarWriter.setCompressionMethod(TSBTarCompressMethod.cmBZip2);

            progressForm = new ProgressDialog();

            progressForm.setCaptionText("Compressing...");
            progressForm.setButtonOKEnabled(false);
            progressForm.setButtonCancelEnabled(true);
            progressForm.setCancelOperation(false);

            progressForm.setVisible(true);

            java.lang.Thread t = new java.lang.Thread(new Runnable() {
                public void run() {
                    try
                    {
                        elTarWriter.compress(saveArchiveName);

                        elTarWriter.setCompressionMethod(oldCM);
                    }
                    catch (Exception ex)
                    {
                        showErrorMessage("Compression error", ex.getMessage());
                    }
                    finally
                    {
                        progressForm.setButtonCancelEnabled(false);
                        progressForm.setButtonOKEnabled(true);
                    }
                    refreshFilesList();
                }
            });

            t.start();
        }
	}
	
	private void saveArchive() {
		
		progressForm = new ProgressDialog();

		progressForm.setCaptionText("Compressing...");
		progressForm.setButtonOKEnabled(false);
		progressForm.setButtonCancelEnabled(true);
		progressForm.setCancelOperation(false);
		
		progressForm.setVisible(true);
		
		java.lang.Thread t = new java.lang.Thread(new Runnable() {
			public void run() {
				try
				{
					if (elTarWriter.getNewArchive())
                        elTarWriter.compress(newArchiveName);
			        else
                        elTarWriter.compress();
				}
				catch (Exception ex)
				{
					showErrorMessage("Compression error", ex.getMessage());
				}
				finally
				{
					progressForm.setButtonCancelEnabled(false);
					progressForm.setButtonOKEnabled(true);
				}
				refreshFilesList();							
			}
		});
		
		t.start();
	}
	

	private void extractSelectedFiles(String outputPath)
	{
		progressForm = new ProgressDialog();
		progressForm.setButtonOKEnabled(false);
		progressForm.setButtonCancelEnabled(true);
		progressForm.setCancelOperation(false);
		progressForm.clearLog();
		progressForm.setCaptionText("Extracting...");
		progressForm.setVisible(true);
		
		extractToPath = outputPath;
		
		java.lang.Thread t = new java.lang.Thread(new Runnable() {
			public void run() {
				try
				{
					ArrayList list = ((KeyTableModel)table.getModel()).getSelectedItems(table.getSelectedRows());
                    elTarWriter.extract(list, extractToPath);
				}
				catch (Exception ex)
				{
					progressForm.addToLog("Error extracting file: " + ex.getMessage());                
				}
				finally
				{
					progressForm.setButtonCancelEnabled(false);
					progressForm.setButtonOKEnabled(true);
				}
			}
		});
		
		t.start();
	}

	private int selectUserAction(String description) {
		int result = JOptionPane.showOptionDialog(null, description, "Warning", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null, new String[] { "Abort", "Retry", "Ignore" }, null);
		
		if (result == 0)
			return SBArcTar.TAR_ACTION_ABORT;
		else if (result == 1)
			return SBArcTar.TAR_ACTION_RETRY;

		return SBArcTar.TAR_ACTION_IGNORE;
	}
	
	// events
	
	TSBTarUserActionNeededEvent.Callback actionClbk = new TSBTarUserActionNeededEvent.Callback() {
		public void tsbTarUserActionNeededEventCallback(TObject sender, int forEvent, String description, TObject param, TSBInteger action) {
			action.value = selectUserAction(description);
		}
	};
	
	TSBTarCompressionFinishedEvent.Callback finishClbk = new TSBTarCompressionFinishedEvent.Callback() {
		public void tsbTarCompressionFinishedEventCallback(TObject sender, TElTarArchiveDirectoryEntry entry) {
			if (!entry.getIsDirectory())
				progressForm.addToLog("File compressed : " + entry.getFileName());
		}
	};
	
	TSBTarCompressionStartEvent.Callback compressionStartClbk = new TSBTarCompressionStartEvent.Callback() {
		public void tsbTarCompressionStartEventCallback(TObject sender, TElTarArchiveDirectoryEntry entry, TSBBoolean allow) {
			progressForm.setCurrentFileName(entry.getPath());
			progressForm.setProgressBarCurrentFile(0);
			allow.value = true;
		}
	};
	
	TSBTarExtractionFinishedEvent.Callback extrClbk = new TSBTarExtractionFinishedEvent.Callback() {
		public void tsbTarExtractionFinishedEventCallback(TObject sender, TElTarArchiveDirectoryEntry entry) {
			if (!entry.getIsDirectory())
				progressForm.addToLog("File extracted : " + entry.getFileName());
		}
	}; 
	
	TSBTarProgressEvent.Callback progressClbk = new TSBTarProgressEvent.Callback() {
		public void tsbTarProgressEventCallback(TObject sender, long processed, long total, long overallProcessed, long overallTotal, TSBBoolean cancel) {
			long processedNorm;
            if (total > 0)
                processedNorm = ((processed * 100000) / total);
            else
                processedNorm = 100000;
			progressForm.setProgressBarCurrentFile((int) processedNorm);
            if (overallTotal > 0)
                processedNorm = (overallProcessed * 100000 / overallTotal);
            else
                processedNorm = 100000;
			progressForm.setProgressBarTotal((int) processedNorm);
			cancel.value = progressForm.getCancelOperation();
		}
	};
	
	TSBTarExtractionStartEvent.Callback extrStartClbk = new TSBTarExtractionStartEvent.Callback() {
		public void tsbTarExtractionStartEventCallback(TObject sender, TElTarArchiveDirectoryEntry entry, TSBBoolean allow) {
			progressForm.setCurrentFileName(entry.getPath());
			progressForm.setProgressBarCurrentFile(0);
			allow.value = true;
		}
	};

    TSBTarUnCompressionTarFromGZipStartEvent.Callback unCompressionGZipStartClbk = new TSBTarUnCompressionTarFromGZipStartEvent.Callback() {
        public void tsbTarUnCompressionTarFromGZipStartEventCallback(TObject sender, TSBBoolean allow) {
            progressForm = new ProgressDialog();

            progressForm.setCaptionText("UnCompressing Tar from GZip...");
            progressForm.setButtonOKEnabled(false);
            progressForm.setButtonCancelEnabled(true);
            progressForm.setCancelOperation(false);

            progressForm.setVisible(true);

            allow.value = true;
        }
    };

    TSBTarUnCompressionTarFromGZipFinishedEvent.Callback unCompressionGZipFinishClbk = new TSBTarUnCompressionTarFromGZipFinishedEvent.Callback() {
        public void tsbTarUnCompressionTarFromGZipFinishedEventCallback(TObject sender) {
            progressForm.setVisible(false);
        }
    };

    TSBTarUnCompressionTarFromBZip2StartEvent.Callback unCompressionBZip2StartClbk = new TSBTarUnCompressionTarFromBZip2StartEvent.Callback() {
        public void tsbTarUnCompressionTarFromBZip2StartEventCallback(TObject sender, TSBBoolean allow) {
            progressForm = new ProgressDialog();

            progressForm.setCaptionText("UnCompressing Tar from BZip2...");
            progressForm.setButtonOKEnabled(false);
            progressForm.setButtonCancelEnabled(true);
            progressForm.setCancelOperation(false);

            progressForm.setVisible(true);

            allow.value = true;
        }
    };

    TSBTarUnCompressionTarFromBZip2FinishedEvent.Callback unCompressionBZip2FinishClbk = new TSBTarUnCompressionTarFromBZip2FinishedEvent.Callback() {
        public void tsbTarUnCompressionTarFromBZip2FinishedEventCallback(TObject sender) {
            progressForm.setVisible(false);
        }
    };

    TSBTarCompressionTarToGZipStartEvent.Callback compressionGZipStartClbk = new TSBTarCompressionTarToGZipStartEvent.Callback() {
        public void tsbTarCompressionTarToGZipStartEventCallback(TObject sender, TSBBoolean allow) {
            progressForm.setCurrentFileName("Compressing Tar to GZip");
            progressForm.setProgressBarCurrentFile(0);
            allow.value = true;
        }
    };

    TSBTarCompressionTarToGZipFinishedEvent.Callback compressionGZipFinishClbk = new TSBTarCompressionTarToGZipFinishedEvent.Callback() {
        public void tsbTarCompressionTarToGZipFinishedEventCallback(TObject sender) {
            progressForm.addToLog("Tar compressed to GZip");
        }
    };

    TSBTarCompressionTarToBZip2StartEvent.Callback compressionBZip2StartClbk = new TSBTarCompressionTarToBZip2StartEvent.Callback() {
        public void tsbTarCompressionTarToBZip2StartEventCallback(TObject sender, TSBBoolean allow) {
            progressForm.setCurrentFileName("Compressing Tar to BZip2");
            progressForm.setProgressBarCurrentFile(0);
            allow.value = true;
        }
    };

    TSBTarCompressionTarToBZip2FinishedEvent.Callback compressionBZip2FinishClbk = new TSBTarCompressionTarToBZip2FinishedEvent.Callback() {
        public void tsbTarCompressionTarToBZip2FinishedEventCallback(TObject sender) {
            progressForm.addToLog("Tar compressed to BZip2");
        }
    };
}